from xml.etree.ElementTree import SubElement

from field import Field
from errors import RecordValidationError


class Record(object):

    def __init__(self, data, **kwargs):
        self._validate(**kwargs)
        self._element = SubElement(data, 'record', **kwargs)

    @staticmethod
    def _has_model(**kwargs):
        if 'model' not in kwargs:
            raise RecordValidationError(
                    '**kwargs has no required key "model"')
        return None

    @staticmethod
    def _has_invalid(**kwargs):
        valid = ['model', 'id', 'context', 'forcecreate']
        invalid = set(kwargs.keys()) - set(valid)
        if invalid:
            raise RecordValidationError(
                    '**kwargs has arguments: %s' % invalid)
        return None

    def _validate(self, **kwargs):
        self._has_model(**kwargs)
        self._has_invalid(**kwargs)
        return None

    def field(self, **kwargs):
        return Field(self._element, **kwargs)

    @property
    def model(self):
        return self._element.attrib['model']

    @model.setter
    def model(self, value):
        self._element.attrib['model'] = value

    @property
    def id(self):
        return self._element.attrib['id']

    @id.setter
    def id(self, value):
        self._element.attrib['id'] = value

    @property
    def context(self):
        return self._element.attrib['context']

    @context.setter
    def context(self, value):
        self._element.attrib['context'] = value

    @property
    def forcecreate(self):
        return self._element.attrib['forcecreate']

    @forcecreate.setter
    def context(self, value):
        self._element.attrib['forcecreate'] = value



from unittest import TestCase
from xml.etree.ElementTree import Element

from mock import patch

from odatoo.odatoo.record import Record
from odatoo.odatoo.field import Field
from odatoo.odatoo.errors import RecordValidationError


class TestRecord(TestCase):

    def setUp(self):
        self.data = Element('test_data')

    @patch('odatoo.odatoo.record.Record._validate')
    def test_init_calls_create(self, mocked_validate):
        mocked_validate.return_value = None
        Record(self.data, id='test_id')
        Record._validate.assert_called_with(id='test_id')

    def test_field_returns_object_of_type_Field(self):
        record = Record(self.data, model='test')
        field = record.field(name='test_name')
        self.assertEqual(type(field), Field)

    def test_has_model_returns_None_if_kwargs_has_argument_model(self):
        record = Record(self.data, model='test')
        result = record._has_model(model='test_model', id=1)
        self.assertEqual(result, None)

    def test_has_model_raises_error_if_kwargs_has_no_argument_model(self):
        record = Record(self.data, model='test')
        with self.assertRaises(RecordValidationError):
            record._has_model(id=1)

    def test_validate_is_None_if_all_arguments_are_valid(self):
        record = Record(self.data, model='test')
        result = record._validate(model='test_model', id=1)
        self.assertTrue(result is None)

    def test_validate_raises_error_if_invalid_arguments(self):
        record = Record(self.data, model='test')
        with self.assertRaises(RecordValidationError):
            record._validate(model='test_model', invalid_arg=1)

    def test_validate_raises_error_if_no_argument_model(self):
        record = Record(self.data, model='test')
        with self.assertRaises(RecordValidationError):
            record._validate(id=1)



from unittest import TestCase
from xml.etree.ElementTree import Element

from mock import patch

from odatoo.odatoo.field import Field
from odatoo.odatoo.errors import FieldValidationError


class TestField(TestCase):

    def setUp(self):
        self.record = Element('test_record')

    @patch('odatoo.odatoo.field.Field._validate')
    def test_init_calls_validate(self, mocked_validate):
        mocked_validate.return_value = None
        Field(self.record, name='test')
        Field._validate.assert_called_with(name='test')

    def test_creating_Field_when_value_is_in_attributes(self):
        field = Field(self.record, name='test_name', value='test_value')
        self.assertEqual(field.value, 'test_value')

    def test_creating_Field_when_no_value_is_in_attributes(self):
        field = Field(self.record, name='test_name')
        self.assertEqual(field.value, None)

    def test_creating_Field_when_value_is_not_str_coverts_to_str(self):
        field = Field(self.record, name='test_name', value=1)
        self.assertEqual(field.value, '1')

    def test_has_name_returns_None_if_kwargs_has_argument_name(self):
        field = Field(self.record, name='test')
        result = field._has_name(name='test')
        self.assertEqual(result, None)

    def test_has_name_raises_error_if_kwargs_has_no_argument_name(self):
        field = Field(self.record, name='test')
        with self.assertRaises(FieldValidationError):
            field._has_name(id=1)

    def test_validate_is_None_if_all_arguments_are_valid(self):
        field = Field(self.record, name='test')
        result = field._validate(name='test')
        self.assertTrue(result is None)

    def test_validate_raises_error_if_invalid_arguments(self):
        field = Field(self.record, name='test')
        with self.assertRaises(FieldValidationError):
            field._validate(name='test_model', invalid_arg=1)

    def test_validate_raises_error_if_no_argument_model(self):
        field = Field(self.record, name='test')
        with self.assertRaises(FieldValidationError):
            field._validate(id=1)



from xml.etree.ElementTree import SubElement

from errors import FieldValidationError


class Field(object):

    def __init__(self, record, **kwargs):
        self._validate(**kwargs)

        if 'value' in kwargs:
            value = str(kwargs['value'])
            del kwargs['value']
        else:
            value = None

        self._element = SubElement(record, 'field', kwargs)
        self.value = value

    @staticmethod
    def _has_name(**kwargs):
        if 'name' not in kwargs:
            raise FieldValidationError(
                    '**kwargs has no required key "name"')
        return None

    @staticmethod
    def _has_invalid(**kwargs):
        valid = ['value', 'name', 'search', 'ref', 'type', 'eval']
        invalid = set(kwargs.keys()) - set(valid)
        if invalid:
            raise FieldValidationError(
                    '**kwargs has arguments: %s' % invalid)
        return None

    def _validate(self, **kwargs):
        self._has_name(**kwargs)
        self._has_invalid(**kwargs)
        return None

    @property
    def value(self):
        return self._element.text

    @value.setter
    def value(self, value):
        self._element.text = value

    @property
    def name(self):
        return self._element.attrib['name']

    @name.setter
    def name(self, value):
        self._element.attrib['name'] = value

    @property
    def ref(self):
        return self._element.attrib['ref']

    @ref.setter
    def ref(self, value):
        self._element.attrib['ref'] = value

    @property
    def eval(self):
        return self._element.attrib['eval']

    @eval.setter
    def eval(self, value):
        self._element.attrib['eval'] = value



import xml.dom.minidom as xmldom

from xml.etree.ElementTree import Element, SubElement, tostring

from record import Record


class Document(object):

    def __init__(self, update='1'):
        self.document = Element('openerp')
        self.data = SubElement(self.document, 'data', {'noupdate': update})

    def record(self, **kwargs):
        return Record(self.data, **kwargs)

    def write(self, file):
        pretty_xml = self._pretty_print()
        with open(file, 'wb') as f:
            f.write(pretty_xml)

    def _pretty_print(self):
        xml_string = tostring(self.document)
        xml_dom = xmldom.parseString(xml_string)
        return xml_dom.toprettyxml()



from setuptools import setup


setup(
    name = 'odatoo',
    packages = ['odatoo', 'tests'],
    version = '0.3',
    description = 'Odoo XML data files library',
    author = 'Will Earp',
    author_email = 'will.earp@icloud.com',
    url = 'https://github.com/wearp/odatoo',
    download_url = 'https://github.com/wearp/odatoo/tarball/0.3',
    keywords = ['odoo', 'xml', 'data', 'files'],
)



import os

from xml.etree.ElementTree import ElementTree
from unittest import TestCase


from odatoo.odatoo.document import Document
from odatoo.odatoo.record import Record


class TestDocument(TestCase):

    def setUp(self):
        self.document = Document()

    def tearDown(self):
        try:
            os.remove("test.xml")
        except OSError:
            pass

    def test_record_returns_object_of_type_Record(self):
        record = self.document.record(model='test')
        self.assertEqual(type(record), Record)

    def test_write_creates_file_and_writes_document(self):
        self.document.write("test.xml")
        tree = ElementTree()
        tree.parse("test.xml")
        root = tree.getroot()
        data = root.find("data")
        self.assertEqual(root.tag, "openerp")
        self.assertEqual(data.tag, "data")

    def test_pretty_print_return_string(self):
        xml = ('<?xml version="1.0" ?>\n<openerp>\n'
               '\t<data noupdate="1"/>\n</openerp>\n')
        pretty_string = self.document._pretty_print()
        self.assertEqual(pretty_string, xml)
